@@ -1,7 +1,7 @@
 use crate::auditwheel::PlatformTag;
 use crate::build_context::{BridgeModel, ProjectLayout};
 use crate::cross_compile::{find_sysconfigdata, parse_sysconfigdata};
-use crate::python_interpreter::{InterpreterKind, MINIMUM_PYTHON_MINOR};
+use crate::python_interpreter::{InterpreterConfig, InterpreterKind, MINIMUM_PYTHON_MINOR};
 use crate::BuildContext;
 use crate::CargoToml;
 use crate::Metadata21;
@@ -67,7 +67,7 @@ pub struct BuildOptions {
 
     /// The python versions to build wheels for, given as the names of the
     /// interpreters. Uses autodiscovery if not explicitly set.
-    #[clap(short, long)]
+    #[clap(short, long, multiple_values = true, multiple_occurrences = true)]
     pub interpreter: Vec<PathBuf>,
 
     /// Which kind of bindings to use. Possible values are pyo3, rust-cpython, cffi and bin
@@ -272,12 +272,25 @@ impl BuildOptions {
             None => PathBuf::from(&cargo_metadata.target_directory).join("wheels"),
         };
 
+        let generate_abi3_import_lib = is_generating_abi3_import_lib(&cargo_metadata)?;
         let interpreter = if self.interpreter.is_empty() {
             // Auto-detect interpreters
-            find_interpreter(&bridge, &[], &target, get_min_python_minor(&metadata21))?
+            find_interpreter(
+                &bridge,
+                &[],
+                &target,
+                get_min_python_minor(&metadata21),
+                generate_abi3_import_lib,
+            )?
         } else {
             // User given list of interpreters
-            find_interpreter(&bridge, &self.interpreter, &target, None)?
+            find_interpreter(
+                &bridge,
+                &self.interpreter,
+                &target,
+                None,
+                generate_abi3_import_lib,
+            )?
         };
 
         let mut rustc_extra_args = self.rustc_extra_args.clone();
@@ -450,6 +463,33 @@ fn has_abi3(cargo_metadata: &Metadata) -> Result<Option<(u8, u8)>> {
     Ok(None)
 }
 
+/// pyo3 0.16.4+ supports building abi3 wheels without a working Python interpreter for Windows
+/// when `generate-abi3-import-lib` feature is enabled
+fn is_generating_abi3_import_lib(cargo_metadata: &Metadata) -> Result<bool> {
+    let resolve = cargo_metadata
+        .resolve
+        .as_ref()
+        .context("Expected cargo to return metadata with resolve")?;
+    for &lib in PYO3_BINDING_CRATES.iter().rev() {
+        let pyo3_packages = resolve
+            .nodes
+            .iter()
+            .filter(|package| cargo_metadata[&package.id].name.as_str() == lib)
+            .collect::<Vec<_>>();
+        match pyo3_packages.as_slice() {
+            [pyo3_crate] => {
+                let generate_import_lib = pyo3_crate
+                    .features
+                    .iter()
+                    .any(|x| x == "generate-abi3-import-lib");
+                return Ok(generate_import_lib);
+            }
+            _ => continue,
+        }
+    }
+    Ok(false)
+}
+
 /// Tries to determine the [BridgeModel] for the target crate
 pub fn find_bridge(cargo_metadata: &Metadata, bridge: Option<&str>) -> Result<BridgeModel> {
     let resolve = cargo_metadata
@@ -505,7 +545,9 @@ pub fn find_bridge(cargo_metadata: &Metadata, bridge: Option<&str>) -> Result<Br
         println!("🔗 Found rust-cpython bindings");
         BridgeModel::Bindings("rust_cpython".to_string(), MINIMUM_PYTHON_MINOR)
     } else {
-        let package = &cargo_metadata[resolve.root.as_ref().unwrap()];
+        let package = cargo_metadata
+            .root_package()
+            .context("Expected cargo to return metadata with root_package")?;
         let targets: Vec<_> = package
             .targets
             .iter()
@@ -577,6 +619,30 @@ fn find_single_python_interpreter(
     Ok(interpreter)
 }
 
+fn find_host_interpreter(
+    bridge: &BridgeModel,
+    interpreter: &[PathBuf],
+    target: &Target,
+    min_python_minor: Option<usize>,
+) -> Result<Vec<PythonInterpreter>> {
+    let interpreters = if !interpreter.is_empty() {
+        PythonInterpreter::check_executables(interpreter, target, bridge)
+            .context("The given list of python interpreters is invalid")?
+    } else {
+        PythonInterpreter::find_all(target, bridge, min_python_minor)
+            .context("Finding python interpreters failed")?
+    };
+
+    if interpreters.is_empty() {
+        if let Some(minor) = min_python_minor {
+            bail!("Couldn't find any python interpreters with version >= 3.{}. Please specify at least one with -i", minor);
+        } else {
+            bail!("Couldn't find any python interpreters. Please specify at least one with -i");
+        }
+    }
+    Ok(interpreters)
+}
+
 /// Finds the appropriate amount for python versions for each [BridgeModel].
 ///
 /// This means all for bindings, one for cffi and zero for bin.
@@ -585,29 +651,28 @@ pub fn find_interpreter(
     interpreter: &[PathBuf],
     target: &Target,
     min_python_minor: Option<usize>,
+    generate_abi3_import_lib: bool,
 ) -> Result<Vec<PythonInterpreter>> {
     match bridge {
         BridgeModel::Bindings(binding_name, _) => {
-            let mut interpreter = if !interpreter.is_empty() {
-                PythonInterpreter::check_executables(interpreter, target, bridge)
-                    .context("The given list of python interpreters is invalid")?
-            } else {
-                PythonInterpreter::find_all(target, bridge, min_python_minor)
-                    .context("Finding python interpreters failed")?
-            };
-
-            if interpreter.is_empty() {
-                if let Some(minor) = min_python_minor {
-                    bail!("Couldn't find any python interpreters with version >= 3.{}. Please specify at least one with -i", minor);
-                } else {
-                    bail!("Couldn't find any python interpreters. Please specify at least one with -i");
+            let mut interpreters = Vec::new();
+            if let Some(config_file) = env::var_os("PYO3_CONFIG_FILE") {
+                if !binding_name.starts_with("pyo3") {
+                    bail!("Only pyo3 bindings can be configured with PYO3_CONFIG_FILE");
                 }
-            }
-
-            if binding_name.starts_with("pyo3") && target.is_unix() && target.cross_compiling() {
+                let interpreter_config =
+                    InterpreterConfig::from_pyo3_config(config_file.as_ref(), target)
+                        .context("Invalid PYO3_CONFIG_FILE")?;
+                interpreters.push(PythonInterpreter::from_config(interpreter_config));
+            } else if binding_name.starts_with("pyo3")
+                && target.is_unix()
+                && target.cross_compiling()
+            {
                 if let Some(cross_lib_dir) = std::env::var_os("PYO3_CROSS_LIB_DIR") {
+                    let host_interpreters =
+                        find_host_interpreter(bridge, interpreter, target, min_python_minor)?;
                     println!("⚠️ Cross-compiling is poorly supported");
-                    let host_python = &interpreter[0];
+                    let host_python = &host_interpreters[0];
                     println!(
                         "🐍 Using host {} for cross-compiling preparation",
                         host_python
@@ -656,32 +721,77 @@ pub fn find_interpreter(
                             }
                         })
                         .context("unsupported Python interpreter")?;
-                    interpreter = vec![PythonInterpreter {
-                        major,
-                        minor,
-                        abiflags,
-                        target: target.clone(),
+                    interpreters.push(PythonInterpreter {
+                        config: InterpreterConfig {
+                            major,
+                            minor,
+                            interpreter_kind,
+                            abiflags,
+                            ext_suffix: ext_suffix.to_string(),
+                            abi_tag,
+                            pointer_width: None,
+                        },
                         executable: PathBuf::new(),
-                        ext_suffix: ext_suffix.to_string(),
-                        interpreter_kind,
-                        abi_tag,
-                        libs_dir: PathBuf::from(cross_lib_dir),
                         platform: None,
                         runnable: false,
-                    }];
+                    });
+                } else {
+                    if interpreter.is_empty() {
+                        bail!("Couldn't find any python interpreters. Please specify at least one with -i");
+                    }
+                    for interp in interpreter {
+                        let python = interp
+                            .file_name()
+                            .context("Invalid python interpreter")?
+                            .to_string_lossy();
+                        let (python_impl, python_ver) =
+                            if let Some(ver) = python.strip_prefix("pypy") {
+                                (InterpreterKind::PyPy, ver)
+                            } else if let Some(ver) = python.strip_prefix("python") {
+                                (InterpreterKind::CPython, ver)
+                            } else {
+                                bail!("Unsupported Python interpreter: {}", python);
+                            };
+                        let (ver_major, ver_minor) = python_ver
+                            .split_once('.')
+                            .context("Invalid python interpreter version")?;
+                        let ver_major = ver_major.parse::<usize>().with_context(|| {
+                            format!(
+                                "Invalid python interpreter major version '{}', expect a digit",
+                                ver_major
+                            )
+                        })?;
+                        let ver_minor = ver_minor.parse::<usize>().with_context(|| {
+                            format!(
+                                "Invalid python interpreter minor version '{}', expect a digit",
+                                ver_minor
+                            )
+                        })?;
+                        let sysconfig = InterpreterConfig::lookup(
+                            target.target_os(),
+                            target.target_arch(),
+                            python_impl,
+                            (ver_major, ver_minor),
+                        )
+                        .context("Failed to find a python interpreter")?;
+                        interpreters.push(PythonInterpreter::from_config(sysconfig.clone()));
+                    }
                 }
+            } else {
+                interpreters =
+                    find_host_interpreter(bridge, interpreter, target, min_python_minor)?;
             }
 
             println!(
                 "🐍 Found {}",
-                interpreter
+                interpreters
                     .iter()
                     .map(ToString::to_string)
                     .collect::<Vec<String>>()
                     .join(", ")
             );
 
-            Ok(interpreter)
+            Ok(interpreters)
         }
         BridgeModel::Cffi => {
             let interpreter = find_single_python_interpreter(bridge, interpreter, target, "cffi")?;
@@ -701,32 +811,49 @@ pub fn find_interpreter(
             // Unfortunately, on windows we need one to figure out base_prefix for a linker
             // argument.
             if target.is_windows() {
-                if let Some(manual_base_prefix) = std::env::var_os("PYO3_CROSS_LIB_DIR") {
+                if env::var_os("PYO3_CROSS_LIB_DIR").is_some() {
                     // PYO3_CROSS_LIB_DIR should point to the `libs` directory inside base_prefix
                     // when cross compiling, so we fake a python interpreter matching it
                     println!("⚠️  Cross-compiling is poorly supported");
                     Ok(vec![PythonInterpreter {
-                        major: *major as usize,
-                        minor: *minor as usize,
-                        abiflags: "".to_string(),
-                        target: target.clone(),
+                        config: InterpreterConfig {
+                            major: *major as usize,
+                            minor: *minor as usize,
+                            interpreter_kind: InterpreterKind::CPython,
+                            abiflags: "".to_string(),
+                            ext_suffix: ".pyd".to_string(),
+                            abi_tag: None,
+                            pointer_width: None,
+                        },
+                        executable: PathBuf::new(),
+                        platform: None,
+                        runnable: false,
+                    }])
+                } else if let Some(interp) = interpreter.get(0) {
+                    println!("🐍 Using {} to generate to link bindings (With abi3, an interpreter is only required on windows)", interp);
+                    Ok(interpreter)
+                } else if generate_abi3_import_lib {
+                    println!("🐍 Not using a specific python interpreter (Automatically generating windows import library)");
+                    // fake a python interpreter
+                    Ok(vec![PythonInterpreter {
+                        config: InterpreterConfig {
+                            major: *major as usize,
+                            minor: *minor as usize,
+                            interpreter_kind: InterpreterKind::CPython,
+                            abiflags: "".to_string(),
+                            ext_suffix: ".pyd".to_string(),
+                            abi_tag: None,
+                            pointer_width: None,
+                        },
                         executable: PathBuf::new(),
-                        ext_suffix: ".pyd".to_string(),
-                        interpreter_kind: InterpreterKind::CPython,
-                        abi_tag: None,
-                        libs_dir: PathBuf::from(manual_base_prefix),
                         platform: None,
                         runnable: false,
                     }])
                 } else {
-                    let interp = interpreter
-                        .get(0)
-                        .context("Failed to find a python interpreter")?;
-                    println!("🐍 Using {} to generate to link bindings (With abi3, an interpreter is only required on windows)", interp);
-                    Ok(interpreter)
+                    bail!("Failed to find a python interpreter");
                 }
             } else {
-                println!("🐍 Not using a specific python interpreter (With abi3, an interpreter is only required on windows)");
+                println!("🐍 Not using a specific python interpreter");
                 Ok(interpreter)
             }
         }
