@@ -44,6 +44,8 @@ pub struct Metadata21 {
     pub maintainer: Option<String>,
     pub maintainer_email: Option<String>,
     pub license: Option<String>,
+    // https://peps.python.org/pep-0639/#license-file-multiple-use
+    pub license_files: Vec<PathBuf>,
     pub classifiers: Vec<String>,
     pub requires_dist: Vec<String>,
     pub provides_dist: Vec<String>,
@@ -144,16 +146,34 @@ impl Metadata21 {
                 }
                 if let Some(license_path) = file {
                     let license_path = manifest_path.join(license_path);
-                    self.license = Some(fs::read_to_string(&license_path).context(format!(
-                            "Failed to read license file specified in pyproject.toml, which should be at {}",
-                            license_path.display()
-                        ))?);
+                    self.license_files.push(license_path);
                 }
                 if let Some(license_text) = text {
                     self.license = Some(license_text.clone());
                 }
             }
 
+            // Until PEP 639 is approved with metadata 2.3, we can assume a
+            // dynamic license-files (also awaiting full 2.2 metadata support)
+            // We're already emitting the License-Files metadata without issue.
+            // license-files.globs = ["LICEN[CS]E*", "COPYING*", "NOTICE*", "AUTHORS*"]
+            let license_include_targets = ["LICEN[CS]E*", "COPYING*", "NOTICE*", "AUTHORS*"];
+            let escaped_manifest_string = glob::Pattern::escape(manifest_path.to_str().unwrap());
+            let escaped_manifest_path = Path::new(&escaped_manifest_string);
+            for pattern in license_include_targets.iter() {
+                for license_path in
+                    glob::glob(&escaped_manifest_path.join(pattern).to_string_lossy())?
+                        .filter_map(Result::ok)
+                {
+                    // if the pyproject.toml specified the license file,
+                    // then we won't list it as automatically included
+                    if !self.license_files.contains(&license_path) {
+                        println!("📦 Including license file \"{}\"", license_path.display());
+                        self.license_files.push(license_path);
+                    }
+                }
+            }
+
             if let Some(authors) = &project.authors {
                 let mut names = Vec::with_capacity(authors.len());
                 let mut emails = Vec::with_capacity(authors.len());
@@ -282,13 +302,16 @@ impl Metadata21 {
 
         let extra_metadata = cargo_toml.remaining_core_metadata();
 
-        let description: Option<String>;
-        let description_content_type: Option<String>;
+        let mut description: Option<String> = None;
+        let mut description_content_type: Option<String> = None;
         // See https://packaging.python.org/specifications/core-metadata/#description
-        if let Some(ref readme) = cargo_toml.package.readme {
+        // and https://doc.rust-lang.org/cargo/reference/manifest.html#the-readme-field
+        if cargo_toml.package.readme == Some("false".to_string()) {
+            // > You can suppress this behavior by setting this field to false
+        } else if let Some(ref readme) = cargo_toml.package.readme {
             let readme_path = manifest_path.as_ref().join(readme);
             description = Some(fs::read_to_string(&readme_path).context(format!(
-                "Failed to read readme specified in Cargo.toml, which should be at {}",
+                "Failed to read Readme specified in Cargo.toml, which should be at {}",
                 readme_path.display()
             ))?);
 
@@ -296,8 +319,22 @@ impl Metadata21 {
                 .description_content_type
                 .or_else(|| Some(path_to_content_type(&readme_path)));
         } else {
-            description = None;
-            description_content_type = None;
+            // > If no value is specified for this field, and a file named
+            // > README.md, README.txt or README exists in the package root
+            // Even though it's not what cargo does, we also search for README.rst
+            // since it's still popular in the python world
+            for readme_guess in ["README.md", "README.txt", "README.rst", "README"] {
+                let guessed_readme = manifest_path.as_ref().join(readme_guess);
+                if guessed_readme.exists() {
+                    let context = format!(
+                        "Readme at {} exists, but can't be read",
+                        guessed_readme.display()
+                    );
+                    description = Some(fs::read_to_string(&guessed_readme).context(context)?);
+                    description_content_type = Some(path_to_content_type(&guessed_readme));
+                    break;
+                }
+            }
         };
         let name = extra_metadata
             .name
@@ -334,6 +371,7 @@ impl Metadata21 {
             author: authors,
             author_email,
             license: cargo_toml.package.license.clone(),
+            license_files: Vec::new(),
 
             // Values provided through `[project.metadata.maturin]`
             classifiers,
@@ -387,6 +425,13 @@ impl Metadata21 {
         add_vec("Requires-External", &self.requires_external);
         add_vec("Provides-Extra", &self.provides_extra);
 
+        let license_files: Vec<String> = self
+            .license_files
+            .iter()
+            .map(|path| path.file_name().unwrap().to_str().unwrap().to_string())
+            .collect();
+        add_vec("License-File", &license_files);
+
         let mut add_option = |name, value: &Option<String>| {
             if let Some(some) = value.clone() {
                 fields.push((name, some));
@@ -485,6 +530,7 @@ fn fold_header(text: &str) -> String {
         if i > 0 {
             result.push_str("\r\n");
         }
+        let line = line.trim_end();
         if line.is_empty() {
             result.push('\t');
         } else {
@@ -505,7 +551,7 @@ mod test {
         let mut readme_md = tempfile::NamedTempFile::new().unwrap();
 
         let readme_path = if cfg!(windows) {
-            readme_md.path().to_str().unwrap().replace("\\", "/")
+            readme_md.path().to_str().unwrap().replace('\\', "/")
         } else {
             readme_md.path().to_str().unwrap().to_string()
         };
@@ -514,7 +560,7 @@ mod test {
 
         let toml_with_path = cargo_toml.replace("REPLACE_README_PATH", &readme_path);
 
-        let cargo_toml_struct: CargoToml = toml::from_str(&toml_with_path).unwrap();
+        let cargo_toml_struct: CargoToml = toml_edit::easy::from_str(&toml_with_path).unwrap();
 
         let metadata =
             Metadata21::from_cargo_toml(&cargo_toml_struct, &readme_md.path().parent().unwrap())
@@ -700,7 +746,7 @@ mod test {
         "#
         );
 
-        let cargo_toml_struct: CargoToml = toml::from_str(&cargo_toml).unwrap();
+        let cargo_toml_struct: CargoToml = toml_edit::easy::from_str(cargo_toml).unwrap();
         let metadata =
             Metadata21::from_cargo_toml(&cargo_toml_struct, "/not/exist/manifest/path").unwrap();
         let actual = metadata.to_file_contents();
@@ -745,7 +791,7 @@ mod test {
     #[test]
     fn test_merge_metadata_from_pyproject_toml() {
         let cargo_toml_str = fs_err::read_to_string("test-crates/pyo3-pure/Cargo.toml").unwrap();
-        let cargo_toml: CargoToml = toml::from_str(&cargo_toml_str).unwrap();
+        let cargo_toml: CargoToml = toml_edit::easy::from_str(&cargo_toml_str).unwrap();
         let metadata = Metadata21::from_cargo_toml(&cargo_toml, "test-crates/pyo3-pure").unwrap();
         assert_eq!(
             metadata.summary,
@@ -773,6 +819,10 @@ mod test {
                 "boltons; (sys_platform == 'win32') and extra == 'test'"
             ]
         );
+        assert_eq!(metadata.license.as_ref().unwrap(), "MIT");
+
+        let license_file = &metadata.license_files[0];
+        assert_eq!(license_file.file_name().unwrap(), "LICENSE");
 
         let content = metadata.to_file_contents();
         let pkginfo: Result<python_pkginfo::Metadata, _> = content.parse();
@@ -783,7 +833,7 @@ mod test {
     fn test_merge_metadata_from_pyproject_toml_with_customized_python_source_dir() {
         let cargo_toml_str =
             fs_err::read_to_string("test-crates/pyo3-mixed-py-subdir/Cargo.toml").unwrap();
-        let cargo_toml: CargoToml = toml::from_str(&cargo_toml_str).unwrap();
+        let cargo_toml: CargoToml = toml_edit::easy::from_str(&cargo_toml_str).unwrap();
         let metadata =
             Metadata21::from_cargo_toml(&cargo_toml, "test-crates/pyo3-mixed-py-subdir").unwrap();
         // defined in Cargo.toml
@@ -794,4 +844,38 @@ mod test {
         // defined in pyproject.toml
         assert_eq!(metadata.scripts["get_42"], "pyo3_mixed_py_subdir:get_42");
     }
+
+    #[test]
+    fn test_implicit_readme() {
+        let cargo_toml_str = fs_err::read_to_string("test-crates/pyo3-mixed/Cargo.toml").unwrap();
+        let cargo_toml = toml_edit::easy::from_str(&cargo_toml_str).unwrap();
+        let metadata = Metadata21::from_cargo_toml(&cargo_toml, "test-crates/pyo3-mixed").unwrap();
+        assert!(metadata.description.unwrap().starts_with("# pyo3-mixed"));
+        assert_eq!(
+            metadata.description_content_type.unwrap(),
+            "text/markdown; charset=UTF-8; variant=GFM"
+        );
+    }
+
+    #[test]
+    fn test_merge_metadata_from_pyproject_dynamic_license_test() {
+        let manifest_path = PathBuf::from("test-crates").join("license-test");
+        let cargo_toml_str = fs_err::read_to_string(&manifest_path.join("Cargo.toml")).unwrap();
+        let cargo_toml: CargoToml = toml_edit::easy::from_str(&cargo_toml_str).unwrap();
+        let metadata = Metadata21::from_cargo_toml(&cargo_toml, &manifest_path).unwrap();
+
+        // verify Cargo.toml value came through
+        assert_eq!(metadata.license.as_ref().unwrap(), "MIT");
+
+        // verify we have the total number of expected licenses
+        assert_eq!(4, metadata.license_files.len());
+
+        // Verify pyproject.toml license = {file = ...} worked
+        assert_eq!(metadata.license_files[0], manifest_path.join("LICENCE.txt"));
+
+        // Verify the default licenses were included
+        assert_eq!(metadata.license_files[1], manifest_path.join("LICENSE"));
+        assert_eq!(metadata.license_files[2], manifest_path.join("NOTICE.md"));
+        assert_eq!(metadata.license_files[3], manifest_path.join("AUTHORS.txt"));
+    }
 }
