@@ -3,7 +3,7 @@ use anyhow::{format_err, Context, Result};
 use pyproject_toml::PyProjectToml as ProjectToml;
 use serde::{Deserialize, Serialize};
 use std::fs;
-use std::path::Path;
+use std::path::{Path, PathBuf};
 
 /// The `[tool]` section of a pyproject.toml
 #[derive(Serialize, Deserialize, Debug, Clone)]
@@ -16,6 +16,7 @@ pub struct Tool {
 #[derive(Serialize, Deserialize, Debug, Clone)]
 #[serde(rename_all = "kebab-case")]
 pub struct ToolMaturin {
+    manifest_path: Option<PathBuf>,
     sdist_include: Option<Vec<String>>,
     bindings: Option<String>,
     cargo_extra_args: Option<String>,
@@ -61,7 +62,7 @@ impl PyProjectToml {
             "Couldn't find pyproject.toml at {}",
             path.display()
         ))?;
-        let pyproject: PyProjectToml = toml::from_str(&contents)
+        let pyproject: PyProjectToml = toml_edit::easy::from_str(&contents)
             .map_err(|err| format_err!("pyproject.toml is not PEP 517 compliant: {}", err))?;
         Ok(pyproject)
     }
@@ -84,6 +85,7 @@ impl PyProjectToml {
             .as_ref()?
             .cargo_extra_args
             .as_deref()
+            .filter(|s| !s.trim().is_empty())
     }
 
     /// Returns the value of `[tool.maturin.compatibility]` in pyproject.toml
@@ -99,6 +101,7 @@ impl PyProjectToml {
             .as_ref()?
             .rustc_extra_args
             .as_deref()
+            .filter(|s| !s.trim().is_empty())
     }
 
     /// Returns the value of `[tool.maturin.skip-auditwheel]` in pyproject.toml
@@ -119,6 +122,16 @@ impl PyProjectToml {
             .unwrap_or_default()
     }
 
+    /// Returns the value of `[tool.maturin.manifest-path]` in pyproject.toml
+    pub fn manifest_path(&self) -> Option<&Path> {
+        self.tool
+            .as_ref()?
+            .maturin
+            .as_ref()?
+            .manifest_path
+            .as_deref()
+    }
+
     /// Having a pyproject.toml without a version constraint is a bad idea
     /// because at some point we'll have to do breaking changes and then source
     /// distributions would break
@@ -187,10 +200,15 @@ mod tests {
             build-backend = "maturin"
 
             [tool.maturin]
-            manylinux = "2010""#,
+            manylinux = "2010"
+            cargo-extra-args = ""
+            rustc-extra-args = "  "
+            "#,
         )
         .unwrap();
         let without_constraint = PyProjectToml::new(without_constraint_dir).unwrap();
         assert!(!without_constraint.warn_missing_maturin_version());
+        assert!(without_constraint.cargo_extra_args().is_none());
+        assert!(without_constraint.rustc_extra_args().is_none());
     }
 }
